##########################################################################
#                                                                        # #
# Licensed Materials - Property of IBM                                   #   #
# Restricted Material of IBM                                             #######
# IBM Rational Test RealTime                                                   #
#  Copyright IBM Corp. 2014,2015 All Rights Reserved.                         #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.            #
#                                                                              #
################################################################################
##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Group Node -*-                                   #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   @FDCFILES      : fdc files generated by all sub nodes.                     #
#   @TIOFILES      : tio files generated by all sub nodes.                     #
#   @RODFILES      : rod files generated by all sub nodes.                     #
#   @NODENAMES     : names of all sub nodes.                                   #
#                                                                              #
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] unless($main::TestRTCC); #environment and global variables.

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";

################################################################################
sub Group
{
  if ($STAGE =~ /r/ )
  {
    return 0 if(($ATU_MERGE_REPORT==0) || (scalar(@NODENAMES) < 2)) ;
    #generate merged coverage result files cio and summary
    my $atu_cio = "";
    my $rod_group = $INTDIR.$PATH_SEPARATOR."G$NAME";
    print &PerlTranslate("Group.pl", "Postprocessing Group...\n");

    if (($ATCMODE) && (!$NO_COV_REPORT)) {
	$atu_cio = $INTDIR.$PATH_SEPARATOR."atu.cio";
	$status = atc_postpro ('', $atu_cio);
	$status && die &PerlTranslate("Group.pl", "Failed.\n");
    }
    #merge RODs and include global coverage report if any
    if(mergeRod($rod_group,$atu_cio,0,\@RODFILES)){
       mergeRod($rod_group,$atu_cio,1,\@RODFILES);
    }
    #generate global xrd and req.xml files
    # try to remove xrd files
    my ($d,$b,$e)=SplitFileName($XRDFILE);
    $d.='/' if($d);
    foreach my $f (@NODENAMES){
       unlink($d.$f.'.xrd');
    }
    #and generate the xrd group 
    genXrdGroup($rod_group,$XRDFILE);
    unlink($atu_cio);
    EndProcess();
  }
  return 0;
}

sub mergeRod
{
my $atu_out= shift;#rod file without ext
my $atu_cio= shift;#cio file with ext
my $mergeMode= shift; #0 means merge at service level, 1 means at test level
my $rod= shift;#rod files array with ext
my $ptu;
my $servName;
my $KLsup=0;
my $KLG='';
my $FP6;
my $FP7;
my %BG;
my %DB;
my $x=0;
my $state=1; #0=>skip lines, 1=>copy, 2=> memo DB part, 3=>memo BG part,
   #read cio file if any
   if($atu_cio ne ''){
      open(IN,$atu_cio) or die "unable to read $atu_cio\n";
      while(<IN>){
         next if(/^R\s*$/);
         $KLG.='KL'.$_ if($x);
         $x=1 if(/^G\s*$/);
      }
      close IN;
   }
   open(OU,">$atu_out.rod") or die "unable to write $atu_out.rod\n";
   foreach my $ro (@$rod){
      print "reading $ro....\n";
      open(IN,$ro) or die "unable to read $ro\n";
      while(<IN>){
	if(/^FP6/){     #FP6 Test
	  $FP6=$_;#print ">>>$FP6\n";
	  next;
	} elsif(/^FP7/){      #FP7 Service
	  $FP7=$_;
	  next;
	} elsif(/^DB/){#Detail summary
	  $state=2;
	  next;
	} elsif(/^KL/){#COV line
	  if($KLG ne ''){
	     print OU $KLG;
	     $KLG='';
	     $KLsup=1;
	  }
	  next if($KLsup);
	  s!\),.*$!\)!;
	} elsif(/^BG/){#summary Global
	  $state=3;
	  next;
	} elsif(/^NS (.+)$/){#New Service
	  #print ">>NS>$1==$servName<\n";
	  #if (not in DB or BG section) && (this is a new Service)
	  #when 2 Rod speaks about same service then take only the first one 
	  if($state<=1){
	     if( $servName eq $1){
	     	#service found multiple times=>coverage on service is wrong and service result must be computed
	     	unless($mergeMode){
	     		#retry merge at test level
	     		close IN;
	     		close OU;
	     		return 1;
	     	}
	     } else {
	     	$state=1;
	     	print OU $FP7 if($FP7);
	     	print OU $FP6 if($FP6);
	     	$FP7='';$FP6='';
	     	$KLsup=1 if($mergeMode);
	     }
	  }
	  $servName = $1;
	  #print ">>NS>$servName<\n";
	} elsif(/^NT /){#New Test
	  unless($state){
		  $state=1 ;
	  }
	     print OU $FP6 if($FP6);
	     $FP6='';
	     $KLsup=0;
	} elsif(/^FX /){#File ptu
	   $ptu=$_ unless ($ptu);
	   die "cannot merge report from multiple PTU files\n" if($ptu ne $_);
	}
	if($state==1){
	   print OU;
	   #print '-' x $x; print ;
	}elsif($state==2){
	   chomp;
	   if(/^(..)\s(\d+)$/){
		$DB{$servName.':'.$1}+=$2;
	   } elsif(/^TF\s+(\w+)\s+(\d+)$/){
		$TF{$servName.':'.$1}+=$2;
	   } elsif(/^(RS)\s+(\w+)\s*$/){
		$DB{$servName.':'.$1}=$2 unless($DB{$servName.':'.$1} eq 'INCORRECT');
	   }
	}elsif($state==3){
	   chomp;
	   if(/^(..)\s(\d+)$/){
		$BG{$1}+=$2;
	   }
	}
      }
      close IN;
      $state=0;#skip first lines of the next file until NS or NT
      $x++;
   }
   $servName='';
   print OU "DB\n";
   foreach my $key (sort keys %DB) {
	my ($s,$k)=split(/:/,$key);
	if($s ne $servName){
	    print OU "NS $s\n";
		$servName=$s;
		foreach my $f (keys %TF) {
		    my ($sf,$fn)=split(/:/,$f);
		    print OU "TF $fn $TF{$f}\n" if($s eq $sf);
		}
	}
	print OU "$k $DB{$key}\n";
   }

   print OU "BG\n";
   foreach my $k (keys %BG) {
	print OU "$k $BG{$k}\n";
   }
   close OU;
   open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Group.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
   print  LOG "ROD $atu_out.rod\n";
   close( LOG );

   return 0;
}

sub genXrdGroup
{
my $atu_out= shift;#rod file without ext
my $atu_xrd= shift;#xrd file with ext

  if ($ATU_SPLIT_REPORT)
  {
    $rtx_opt = $rtx_opt . " -s" . $ATU_SPLIT_REPORT;
  }

  # Translator execution
  unlink ( "$atu_xrd" );

  my $headFile="$NAME.summary.txt";
  my $headOption;
  #The following line describe the report header content
  #lines are split on = sign to get 2 columns
  #line feed can be inserted using the 4 following chars: <br>
  #look for REPORTHEADER into the TDPeditor online help for more details
  if(open (HDR,">$headFile")){
     if($RTRT_ACTIVATED){ #=1 on eclipse
	print HDR "Project=$PROJECT_NAME\n" if($PROJECT_NAME ne '');
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_PATH\n" if($WORKSPACE_NAME ne '');
	print HDR "Working directory=$WORKING_DIR\n";
	print HDR "Test name=$NAME\n";
	print HDR "Report file=$atu_xrd\n";
	print HDR "\$PREPROCESS\n";
	print HDR "\$HEADERLINE\n";
     }else{
	print HDR "Project=$PROJECT_NAME\n" if($PROJECT_NAME ne '');
	print HDR "Project file=$PROJECT_PATH$PATH_SEPARATOR$PROJECT_NAME.rtp\n" if($PROJECT_NAME ne '');
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_NAME\n" if($WORKSPACE_NAME ne '');
	print HDR "Working directory=$WORKING_DIR\n";
	print HDR "Test node=$NAME\n";
	print HDR "Report file=".absolutePath($atu_xrd)."\n";
	print HDR "\$PREPROCESS\n"; # print the preprocessor command line in this place
	print HDR "\$HEADERLINE\n"; # print the test script header parameters in this place
     }
#	print HDR "=$\n";
     if(($FILEVERCMS) && ($FILEVERCMS ne '0'))
     {
        print &PerlTranslate("UnitTesting.pl", "Check file's version\n", "");
        print HDR "#Tested files version\n";
 	foreach my $f (@SOURCES) #(@INTEGRATEDFILENAMES,@ADDITONALFILES)
 	{
 		my $v=getFileVersion($f);
		$f=absolutePath($f);
 		print HDR "Tested File:$f=Version:$v\n";
	}
     }
     close HDR;
     $headOption=" \"-h$headFile\"";
  }

  if($Ini{'REPORTHEADER'}) {
     print "REPORTHEADER specified as:$LP_TARGETDIR/cmd/$Ini{'REPORTHEADER'}\n";
     print "sub reportHeader($headFile,$RTRT_ACTIVATED)\n";
     require "$TARGETDIR/cmd/$Ini{'REPORTHEADER'}";
     reportHeader($headFile,$RTRT_ACTIVATED);
  }

  my $atu_command = $BINPATH."rod2xrd" . $rtx_opt . $headOption. " \"-l$LOGFILENAME\"" . " \"-o$atu_xrd\" \"$atu_out.rod\"" ;
  my $trans_status = SystemP ($atu_command);
  $trans_status && die &PerlTranslate("UnitTesting.pl", "Xrd Translation Failed.\n");

  my ($reqXmld,$reqXml,$e)=SplitFileName($atu_xrd);$reqXml=$reqXmld.$reqXml.'.req.xml';
  my $req_command = $BINPATH."rod2req" . $headOption. " \"-l$LOGFILENAME\"" . " \"-o$reqXml\" \"$atu_out.rod\" -verbose" ;
  my $req_status = SystemP ($req_command);
  $req_status && print &PerlTranslate("UnitTesting.pl", "Requirement generation Failed.\n");
  copy($ENV{TESTRTDIR}.'/lib/reports/testreq_report.xsl',$reqXmld.'testreq_report.xsl'); #use $ENV for cmd line scripts (TestRT_test)

  Cat($headFile,"$NAME.atc.txt") if(($ATCMODE) && (-f "$NAME.atc.txt"));
  RegisterTmpFiles($headFile);
}
################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

CheckIntDir();
CheckOutDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
exit (Group());
}
1;
# That's all Folks.
################################################################################








